﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior {
  using HeuristicLab.Data;
  using HeuristicLab.Problems.TravelingSalesman;

  /// <summary>
  /// An operator that analyzes schemata.
  /// </summary>
  [Item("BestIndividualSchemataCreator", "An operator that analyzes schemata.")]
  [StorableClass]
  public sealed class BestIndividualSchemataCreator : SingleSuccessorOperator, IAnalyzer {
    private const string ResultsParameterName = "Results";
    private const string SchemataParameterName = "BestIndividualSchemata";
    private const string BestSolutionParameterName = "Best TSP Solution";

    #region IAnalyzer Members
    public bool EnabledByDefault {
      get { return true; }
    }
    #endregion

    #region Parameter properties
    public ILookupParameter<ResultCollection> ResultsParameter {
      get { return (ILookupParameter<ResultCollection>)Parameters[ResultsParameterName]; }
    }
    #endregion

    #region Properties
    public ResultCollection Results {
      get { return ResultsParameter.ActualValue; }
    }
    public PathTSPTour BestSolution {
      get { return ((PathTSPTour)ResultsParameter.ActualValue[BestSolutionParameterName].Value); }
    }
    #endregion

    [StorableConstructor]
    private BestIndividualSchemataCreator(bool deserializing) : base(deserializing) { }
    private BestIndividualSchemataCreator(BestIndividualSchemataCreator original, Cloner cloner) : base(original, cloner) { }
    public BestIndividualSchemataCreator()
      : base() {
      Parameters.Add(new LookupParameter<ResultCollection>(ResultsParameterName, "The results collection where the analysis values should be stored."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new BestIndividualSchemataCreator(this, cloner);
    }

    public override IOperation Apply() {
      var subtours = AlgorithmBehaviorHelpers.ExtractSubtours(BestSolution.Permutation);
      subtours.Add(BestSolution.Permutation);
      Results.Add(new Result(SchemataParameterName, new ItemArray<IntArray>(subtours)));
      return base.Apply();
    }
  }
}
